@@ -1,4 +1,4 @@
-#!/usr/bin/env python
+#!/usr/bin/python
 
 ## Copyright (C) 2008, 2009, 2010 Red Hat, Inc.
 ## Authors:
@@ -16,105 +16,14 @@
 
 ## You should have received a copy of the GNU General Public License
 ## along with this program; if not, write to the Free Software
-## Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+## Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 
 from gettext import gettext as _
 import cupshelpers
 import urllib
-import re, subprocess, time
-from debug import *
+
 import ppdippstr
 
-dnssdbrowsemap = None
-
-def dnssdclearcache():
-    global dnssdbrowsemap
-    dnssdbrowsemap = None
-
-def dnssdresolve(name):
-    global dnssdbrowsemap
-    def expandhex (searchres):
-        expr = searchres.group(0)
-        return chr(int(expr[1:], 16))
-    def expanddec (searchres):
-        expr = searchres.group(0)
-        return chr(int(expr[1:], 10))
-    ip = None
-    dnssdhost = None
-    name = name.encode('utf-8')
-    name = re.sub("%(?i)[\dabcdef]{2}", expandhex, name)
-    if (dnssdbrowsemap == None):
-        for mode in ("-t", "-c"):
-            debugprint ("Browsing DNS-SD hosts ...")
-            cmd = ["avahi-browse", "-a", "-r", mode, "-k", "-p"]
-            try:
-                debugprint ("Running avahi-browse -a -r %s -k -p" % mode)
-                p = subprocess.Popen (cmd, shell=False,
-                                      stdin=file("/dev/null"),
-                                      stdout=subprocess.PIPE,
-                                      stderr=file("/dev/null"))
-                timeout = 2.0
-                starttime = time.time ()
-                debugprint ("Timeout %s sec" % timeout)
-                while p.poll () == None and \
-                      time.time () < starttime + timeout:
-                    time.sleep (0.1)
-                if p.poll () == None:
-                    p.kill ()
-                    debugprint ("avahi-browse timed out, killed.")
-                haveoutput = 0
-                for line in p.stdout:
-                    res = re.search ("^=;[^;]*;[^;]*;([^;]*);[^;]*;[^;]*;([^;]*);([^;]*);",
-                                     line)
-                    if res:
-                        haveoutput = 1
-                        resg = res.groups ()
-                        servicename = resg[0]
-                        dnssdhost = resg[1]
-                        ip = resg[2]
-                        if servicename and (dnssdhost or ip):
-                            servicename = servicename.encode('utf-8')
-                            servicename = re.sub('\\\\\d{3}',
-                                                 expanddec, servicename)
-                            if dnssdhost:
-                                dnssdhost = re.sub('\.local$', '', dnssdhost)
-                                dnssdhost = dnssdhost.encode('utf-8')
-                                dnssdhost = re.sub('\\\\\d{3}',
-                                                   expanddec, dnssdhost)
-                            debugprint ("Found host: Service name: %s; Name: %s; IP: %s" % (servicename, dnssdhost, ip))
-                            if dnssdbrowsemap == None:
-                                dnssdbrowsemap = []
-                            found = 0
-                            for entry in dnssdbrowsemap:
-                                if entry['servicename'] == servicename:
-                                    found = 1
-                                    if ip: entry['ip'] = ip
-                                    if dnssdhost: entry['dnssdhost'] = dnssdhost
-                                    break
-                            if not found:
-                                entry = {};
-                                entry['servicename'] = servicename
-                                if ip: entry['ip'] = ip
-                                if dnssdhost: entry['dnssdhost'] = dnssdhost
-                                dnssdbrowsemap.append(entry)
-                if haveoutput:
-                    break
-            except:
-                # Problem executing command.
-                pass
-        debugprint ("avahi-browse completed, host table %s" % dnssdbrowsemap)
-
-    debugprint ("Resolving host %s ..." % name)
-    if dnssdbrowsemap != None:
-        for entry in dnssdbrowsemap:
-            if entry['servicename'] == name:
-                ip = entry['ip']
-                dnssdhost = entry['dnssdhost']
-                break
-
-    debugprint ("Resolved host: Service name: %s; Name: %s; IP: %s" % (name, dnssdhost, ip))
-    return ip, dnssdhost
-
 class PhysicalDevice:
     def __init__(self, device):
         self.devices = None
@@ -125,7 +34,7 @@ class PhysicalDevice:
         self._ppdippstr = ppdippstr.backends
 
     def _canonical_id (self, device):
-        if device.id_dict:
+        if hasattr (device, "id_dict"):
             mfg = device.id_dict.get ('MFG', '')
             mdl = device.id_dict.get ('MDL', '')
 
@@ -134,7 +43,7 @@ class PhysicalDevice:
             else:
                 make_and_model = "%s %s" % (mfg, mdl)
         else:
-            make_and_model = device.make_and_model
+             make_and_model = device.make_and_model
 
         return cupshelpers.ppds.ppdMakeModelSplit (make_and_model)
 
@@ -156,14 +65,9 @@ class PhysicalDevice:
             else:
                 return None, None
         elif scheme == 'dnssd' or scheme == 'mdns':
-            (hostport, rest) = urllib.splithost (rest)
-            p = hostport.find(".")
-            if (p != -1):
-               hostport = hostport[:p]
-            if hostport == '':
-                return None, None
-            if not uri.endswith("/cups"):
-                (hostport, dnssdhost) = dnssdresolve(hostport);
+            # The URIs of the CUPS "dnssd" backend do not contain the host
+            # name of the printer
+            return None, None
         else:
             (hostport, rest) = urllib.splithost (rest)
             if hostport == None:
@@ -176,11 +80,14 @@ class PhysicalDevice:
     def add_device (self, device):
         if self._network_host or self.dnssd_hostname:
             host, dnssdhost = self._get_host_from_uri (device.uri)
-            if (host == None and hasattr (device, 'address')):
+            if (hasattr (device, 'address')):
                 host = device.address
+            if (hasattr (device, 'hostname') and dnssdhost == None):
+                dnssdhost = device.hostname
             if (host == None and dnssdhost == None) or \
-               (host and self._network_host and host != self._network_host) or \
-               (dnssdhost and self.dnssd_hostname and
+               (host and self._network_host and \
+                host != self._network_host) or \
+               (dnssdhost and self.dnssd_hostname and \
                 dnssdhost != self.dnssd_hostname) or \
                (host == None and self.dnssd_hostname == None) or \
                (dnssdhost == None and self._network_host == None):
@@ -232,14 +139,20 @@ class PhysicalDevice:
             if dnssdhost:
                 self.dnssd_hostname = dnssdhost;
 
+        if (hasattr (device, 'address') and self._network_host == None):
+            if device.address:
+                self._network_host = device.address
+        if (hasattr (device, 'hostname') and self.dnssd_hostname == None):
+            if device.hostname:
+                self.dnssd_hostname = device.hostname
+
     def get_devices (self):
         return self.devices
 
     def get_info (self):
         # If the manufacturer/model is not known, or useless (in the
         # case of the hpfax backend or a dnssd URI pointing to a remote
-        # CUPS queue), show the device-info field
-        # instead.
+        # CUPS queue), show the device-info fiel instead.
         if self.mfg == '' or \
            (self.mfg == "HP" and self.mdl.startswith("Fax")) or \
            ((self.devices[0].uri.startswith('dnssd:') or \
@@ -248,9 +161,7 @@ class PhysicalDevice:
             info = self._ppdippstr.get (self.devices[0].info)
         else:
             info = "%s %s" % (self.mfg, self.mdl)
-        if len (self.sn) > 0:
-            info += " (%s)" % self.sn
-        elif ((self._network_host and len (self._network_host) > 0) or \
+        if ((self._network_host and len (self._network_host) > 0) or \
               (self.dnssd_hostname and len (self.dnssd_hostname))) and not \
              ((self.devices[0].uri.startswith('dnssd:') or \
                self.devices[0].uri.startswith('mdns:')) and \
@@ -261,6 +172,8 @@ class PhysicalDevice:
                 info += " (%s)" % self.dnssd_hostname
             else:
                 info += " (%s, %s)" % (self.dnssd_hostname, self._network_host)
+        elif len (self.sn) > 0:
+            info += " (%s)" % self.sn
         return info
 
     # User data
