@@ -16,7 +16,7 @@
 
 ## You should have received a copy of the GNU General Public License
 ## along with this program; if not, write to the Free Software
-## Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+## Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 
 import asyncconn
 import authconn
@@ -774,6 +774,11 @@ class JobViewer (GtkGUI):
     def add_job (self, job, data, connection=None):
         self.update_job (job, data, connection=connection)
 
+        # There may have been an error fetching additional attributes,
+        # in which case we need to give up.
+        if not self.jobs.has_key (job):
+            return
+
         store = self.store
         iter = self.store.append (None)
         store.set_value (iter, 0, job)
@@ -1441,7 +1446,7 @@ class JobViewer (GtkGUI):
             if jobid not in self.jobs_attrs:
                 # add new notebook page with scrollable treeview
                 scrolledwindow = gtk.ScrolledWindow()
-                label = gtk.Label(jobid) # notebook page has label with jobid
+                label = gtk.Label(str(jobid)) # notebook page has label with jobid
                 page_index = self.notebook.append_page(scrolledwindow, label)
                 attr_treeview = gtk.TreeView()
                 scrolledwindow.add(attr_treeview)
@@ -1490,7 +1495,7 @@ class JobViewer (GtkGUI):
             for name, value in attrs.iteritems():
                 if name in ['job-id', 'job-printer-up-time']:
                     continue
-                attr_store.append([name, value])
+                attr_store.append([name, str(value)])
 
     def job_is_active (self, jobdata):
         state = jobdata.get ('job-state', cups.IPP_JOB_CANCELED)
@@ -1778,6 +1783,10 @@ class JobViewer (GtkGUI):
         if not self.jobiters.has_key (jobid):
             self.add_job (jobid, jobdata)
 
+        # If we failed to get required attributes for the job, bail.
+        if not self.jobiters.has_key (jobid):
+            return
+
         if self.job_is_active (jobdata):
             self.active_jobs.add (jobid)
         elif jobid in self.active_jobs:
@@ -2062,6 +2071,7 @@ class JobViewer (GtkGUI):
             tuple = reason.get_tuple ()
         else:
             debugprint ("Couldn't find state reason in list!")
+            tuple = None
             for (level,
                  p,
                  r) in self.state_reason_notifications.keys ():
@@ -2070,6 +2080,11 @@ class JobViewer (GtkGUI):
                     tuple = (level, p, r)
                     break
 
+            if tuple == None:
+                debugprint ("Unexpected now_connected signal "
+                            "(reason not in notifications list)")
+                return
+
         try:
             notification = self.state_reason_notifications[tuple]
         except KeyError:
